/* gtkGL
 * Copyright (C) 1997,1998 AOSASA Shigeru
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gtkGLcontext.h"

#include <GL/gl.h>

#include <gdk/gdk.h>
#include <gdkGL/gdkGL.h>

#include <gtk/gtksignal.h>
#include <gtk/gtkwidget.h>


enum {
  ACTIVATE_NOTIFY,
  INACTIVATE_NOTIFY,
  LAST_SIGNAL
};


static void gtk_gl_context_class_init (GtkGLContextClass *klass);
static void gtk_gl_context_init       (GtkGLContext      *gtk_gl_context);
static void gtk_gl_context_finalize   (GtkObject         *object);

static void gtk_gl_context_activate_notify   (GtkGLContext *context);
static void gtk_gl_context_inactivate_notify (GtkGLContext *context);


static GtkDataClass *parent_class = NULL;
static guint gl_context_signals[LAST_SIGNAL] = { 0 };

static GtkGLContext *gtk_gl_real_active_context = NULL;
static GtkGLContext *gtk_gl_active_context = NULL;


guint
gtk_gl_context_get_type ()
{
  static guint gtk_gl_context_type = 0;

  if (!gtk_gl_context_type)
    {
      GtkTypeInfo gtk_gl_context_info =
      {
	"GtkGLContext",
	sizeof (GtkGLContext),
	sizeof (GtkGLContextClass),
	(GtkClassInitFunc) gtk_gl_context_class_init,
	(GtkObjectInitFunc) gtk_gl_context_init,
	(GtkArgSetFunc) NULL,
	(GtkArgGetFunc) NULL,
      };

      gtk_gl_context_type = gtk_type_unique (gtk_data_get_type (),
					     &gtk_gl_context_info);
    }

  return gtk_gl_context_type;
}

static void
gtk_gl_context_class_init (GtkGLContextClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;

  parent_class = gtk_type_class (gtk_data_get_type ());

  gl_context_signals[ACTIVATE_NOTIFY] =
      gtk_signal_new ("activate_notify",
		      GTK_RUN_FIRST,
		      object_class->type,
		      GTK_SIGNAL_OFFSET (GtkGLContextClass,
					 activate_notify),
		      gtk_signal_default_marshaller,
		      GTK_TYPE_NONE, 0);
  gl_context_signals[INACTIVATE_NOTIFY] =
      gtk_signal_new ("inactivate_notify",
		      GTK_RUN_FIRST,
		      object_class->type,
		      GTK_SIGNAL_OFFSET (GtkGLContextClass,
					 inactivate_notify),
		      gtk_signal_default_marshaller,
		      GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, gl_context_signals, LAST_SIGNAL);

  object_class->finalize = gtk_gl_context_finalize;

  class->activate_notify = NULL;
  class->inactivate_notify = NULL;
}

static void
gtk_gl_context_init (GtkGLContext *gtk_gl_context)
{
  gtk_gl_context->gdk_gl_context = NULL;
  gtk_gl_context->owner_widget = NULL;
}

GtkObject*
gtk_gl_context_new (void)
{
  GtkGLContext *gtk_gl_context;

  gtk_gl_context = gtk_type_new (gtk_gl_context_get_type ());

  return GTK_OBJECT (gtk_gl_context);
}

void
gtk_gl_context_set_owner (GtkGLContext *context,
			  GtkWidget    *widget)
{
  g_return_if_fail (context != NULL);
  g_return_if_fail (GTK_IS_GL_CONTEXT (context));
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_WIDGET (widget));
  g_return_if_fail (context->owner_widget == NULL);

  context->owner_widget = widget;

  gtk_object_ref (GTK_OBJECT (context));
  gtk_object_sink (GTK_OBJECT (context));
}

GtkWidget*
gtk_gl_context_get_owner (GtkGLContext *context)
{
  g_return_val_if_fail (context != NULL,             NULL);
  g_return_val_if_fail (GTK_IS_GL_CONTEXT (context), NULL);

  return (context->owner_widget);
}

void
gtk_gl_context_realize (GtkGLContext *context)
{
  GdkGLContext *gdk_gl_context;

  g_return_if_fail (context != NULL);
  g_return_if_fail (GTK_IS_GL_CONTEXT (context));
  g_return_if_fail (context->owner_widget != NULL);
  g_return_if_fail (GTK_IS_WIDGET (context->owner_widget));
  g_return_if_fail (GTK_WIDGET_REALIZED (context->owner_widget));
  g_return_if_fail (context->gdk_gl_context == NULL);

  gdk_gl_context
      = gdk_gl_context_new (gtk_widget_get_visual (context->owner_widget));
  gdk_gl_context->user_data = context;

  context->gdk_gl_context = gdk_gl_context;
}



static void
gtk_gl_context_finalize (GtkObject *object)
{
  GtkGLContext *context;

  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_GL_CONTEXT (object));

  context = GTK_GL_CONTEXT (object);

  if (gtk_gl_real_active_context == context)
    gtk_gl_real_active_context = NULL;
  if (gtk_gl_active_context == context)
    gtk_gl_active_context = NULL;

  if (context->gdk_gl_context)
    {
      gdk_gl_context_unref (context->gdk_gl_context);
    }

  (* GTK_OBJECT_CLASS (parent_class)->finalize) (GTK_OBJECT (context));
}

static void
gtk_gl_context_activate_notify (GtkGLContext *gtk_gl_context)
{
  gtk_signal_emit(GTK_OBJECT (gtk_gl_context),
		  gl_context_signals[ACTIVATE_NOTIFY]);
}

static void
gtk_gl_context_inactivate_notify (GtkGLContext *gtk_gl_context)
{
  gtk_signal_emit(GTK_OBJECT (gtk_gl_context),
		  gl_context_signals[INACTIVATE_NOTIFY]);
}



gint
gtk_gl_set_active_context (GtkGLContext *context)
{
  gint r;

  g_return_val_if_fail (context != NULL,                FALSE);
  g_return_val_if_fail (GTK_IS_GL_CONTEXT (context),    FALSE);
  g_return_val_if_fail (!GTK_OBJECT_FLOATING (context), FALSE);
  g_return_val_if_fail (context->owner_widget != NULL,               FALSE);
  g_return_val_if_fail (GTK_IS_WIDGET (context->owner_widget),       FALSE);
  g_return_val_if_fail (GTK_WIDGET_REALIZED (context->owner_widget), FALSE);


  if (context == gtk_gl_real_active_context)
    {
      gtk_gl_active_context = context;
      r = TRUE;
    }
  else
    {
      if (gtk_gl_active_context != NULL)
	  gtk_gl_context_inactivate_notify (gtk_gl_active_context);

      if (context->gdk_gl_context == NULL)
	gtk_gl_context_realize (context);

      r = gdk_gl_set_current (context->gdk_gl_context,
			      context->owner_widget->window);

      if (r == TRUE)
	{
	  gtk_gl_real_active_context = context;
	  gtk_gl_active_context = context;
	  gtk_gl_context_activate_notify (context);
	}
      else
	{
	  gtk_gl_active_context = NULL;
	}
    }

  return r;
}

GtkGLContext *
gtk_gl_get_active_context (void)
{
  return gtk_gl_active_context;
}

void
gtk_gl_unset_active_context (void)
{
  if (gtk_gl_active_context != NULL)
    gtk_gl_context_inactivate_notify (gtk_gl_active_context);

  gtk_gl_active_context = NULL;
}
